# SBT-Depth
The SBT tracker for the VOT2022-Depth challenge

## Install the environment
Environment Requirements:
- Operating System: Ubuntu 20.04
- CUDA: 11.3 is tested and GCC: 9.4.0 is tested
- Dependencies: All dependencies are specified in `install.sh`
- Device: *Intel(R) Core(TM) i7-10700K CPU @ 3.80GHz* and *GeForce RTX 3060*

**Environment Setup**: Use the Anaconda
```
# create environment
conda create -n sbt python=3.8
conda activate sbt

# install pytorch 
pip install torch==1.10.0+cu113 torchvision==0.11.0+cu113 torchaudio==0.10.0 -f https://download.pytorch.org/whl/torch_stable.html

# install dependancies and download model files
bash install.sh

# set up paths for the project
python tracking/create_default_local_file.py --workspace_dir . --data_dir ./data --save_dir .
```
## Download the model from BaiduNetDisk

- SBTv1_base_ep0533.pth.tar：[BaiduNetDisk](https://pan.baidu.com/s/1iVYDyfsDWMKOA-dka9Hzfw), code: h0c0

  move SBTv1_base_ep0533.pth.tar to folder:  checkpoints/SBT/

**Note:If the [install.sh](install.sh) cannot download files, please refer to these links:**
- STARKST_ep0500.pth.tar: [BaiduNetDisk](https://pan.baidu.com/s/12h8BFRICTIgzDnq2e2JPYg), code: dm4v
- SEcmnet_ep0040-a.pth.tar: [BaiduNetDisk](https://pan.baidu.com/s/1aMe4XirCTMn2UJs4ld3f5A), code: h4ll
- DeT_DiMP50_Mean.pth: [BaiduNetDisk](https://pan.baidu.com/s/19XMLy6XXZLYhxuBeITqQpg), code: uqsb

  move STARKST_ep0500.pth.tar to folder: checkpoints/train/stark_ref/baseline/

  move SEcmnet_ep0040-a.pth.tar to folder: checkpoints/ARcm_r34/

  move DeT_DiMP50_Mean.pth to folder: pytracking/networks/


## Test and evaluate SBT on benchmarks

**Workspace**

Our workspace is in folder VOT22/, please refer to our [trackers.ini](VOT22/trackers.ini)


**VOT2022-Depth**
- run the folloing script.
```
cd VOT22
bash exp.sh
```
Note: The <PROJECT_DIR> in [trackers.ini](VOT22/trackers.ini) requires absolute path 
  of the SBT project on your local machine. Our [exp.sh](VOT22/exp.sh) can automatically locate the
  <PROJECT_DIR>. However, you can also manually specify <PROJECT_DIR> in [trackers.ini](VOT22/trackers.ini)


## Runtime Tips (Important)
When the PrRoIPooling module *is compiled for the first time* or *the environment is changed*, the following exception may be raised.

    `[1/3] /usr/local/cuda-11.3/bin/nvcc......SBT_RGBD/ltr/external/PreciseRoIPooling/pytorch/prroi_pool/src/prroi_pooling_gpu.c: In function`

Please **try again**, or **opening a new terminal** or **deactive then re-active the conda environment**.


## Acknowledge
Our code is implemented based on the following libraries:
* [PyTracking](https://github.com/visionml/pytracking)
* [Stark](https://github.com/researchmm/Stark)
* [Timm](https://github.com/rwightman/pytorch-image-models)
